!> \file flottab2-0.5-heino.f90
!! 
!!  
!<

!> SUBROUTINE: massb_perturb_Tparam 
!! Cette routine determine les endroits ou la glace
!! flotte , les streams et la position du front de glace
!! \author Vincent et Catherine
!! \date 10 juillet 2005
!! @note Il y a 4 sortes de zone
!! @note  - Pose
!! @note  - Grounding zone et streams    gzmx et gzmy
!! @note  - Iles             ilemx, ilemy
!! @note  - flottant  flot sur le noeud majeur, flotmx sur le noeud mineur 
!! @note Used modules:
!! @note    - use module3D_phy
!! @note    - use module_choix
!<



! =======================================================
!
       SUBROUTINE FLOTTAB()
!
!       				Vince 5 Jan 95
!                                       Modifie 20 Jan 95
!                                       Modifie le 30 Novembre 98
!    Passage f90 + determination des fronts Vincent dec 2003
!    nettoyage et nouvelle dtermination de gzmx et gzmy Cat le 10 juillet 2005
!
!                                       -----------------
!
!      Cette routine determine les endroits ou la glace
!      flotte , les streams et la position du front de glace
!
!      Il y a 4 sortes de zone
!      Pose
!      Grounding zone et streams    gzmx et gzmy
!      Iles             ilemx, ilemy
!      flottant  flot sur le noeud majeur, flotmx sur le noeud mineur

!   passage dans flottab tous les pas de temps dt  :
! 
! QUESTION : que se passe t'il si un noeud passe en flottant entre 2 dtt
!
! =======================================================

       USE module3D_phy
!       use printtable
!       use sliding_heino
       use module_choix

 IMPLICIT NONE

 real ::  surnet !surnet hauteur de glace au dessus de la mer
 real ::  archim ! test de flottaison

 logical ::  gz1
 logical ::  fl1


 real,dimension(nx,ny) ::  uxs1   ! uxbar a l'entree de flottab
 real,dimension(nx,ny) ::  uys1   ! uybar a l'entree de flottab


 integer pmx,pmy !pm=plus-moins -1 ou 1 pour x et y 


! Variables pour la determination des differents shelfs/stream
! (represents comme des taches ou l'on resoud l'eq elliptique)
!________________________________________________________________
 integer,parameter               :: n_ta_max=2000! nombre de tache max
 integer,dimension(0:nx,0:ny)    :: table_out    ! pour les numeros des taches
 integer,dimension(n_ta_max)     :: compt        !contient les equivalence entre les taches
 integer,dimension(0:n_ta_max)   :: nb_pts_tache ! indique le nombre de points par tache
 logical,dimension(0:n_ta_max)   :: iceberg      ! T si iceberg, F si calotte pose

!  Variables pour determiner le point le plus haut (surf)
!  d'une ile completement stream
!_________________________________________________________

! icetrim : T si ice stream, F si calotte pose(vertical shear)
 logical,dimension(0:2000) :: icetrim 

 integer ::  ii,jj
 integer ::  smax_i
 integer ::  smax_j
 real    ::  smax_


!_________________________________________________________
 if (itracebug.eq.1)  call tracebug(' Heino: entree dans Flottab')



 SHELFY = .FALSE.


! 1-INITIALISATION
! ----------------
! initialisation des variables pour detecter les points qui se mettent
! a flotter entre 2 dtt

       appel_new_flot=.false.
       do j=1,ny
          do i=1,nx
             new_flot_point(i,j)=.false.
             new_flotmx(i,j)=.false.
             new_flotmy(i,j)=.false.
          enddo
       enddo

! ICE(:,:)=(H(:,:).gt.1) ! ice=.true. si epaisseur > 1m

      ICE(:,:)=0
      front(:,:)=0
      frontfacex(:,:)=0
      frontfacey(:,:)=0
      isolx(:,:)=.FALSE.
      isoly(:,:)=.FALSE.
      boost=.false.

! fin de l'initialisation
!_____________________________________________________________________

! 2-TESTE LES NOUVEAUX POINTS FLOTTANTS
! -------------------------------------

 do j=1,ny
   do i=1,nx

      uxs1(i,j)=uxbar(i,j)
      uys1(i,j)=uybar(i,j)

      archim = Bsoc(i,j)+H(i,j)*ro/row -sealevel_2d(i,j)
 
arch: if ((ARCHIM.LT.0.).and.(H(I,J).gt.1.E-3)) then    ! le point flotte

ex_pose: if ((.not.FLOT(I,J)).and.(isynchro.eq.1)) then  !  il ne flottait pas avant
            FLOT(I,J)=.true.
            BOOST=.false.


             if (igrdline.eq.1) then   ! en cas de grounding line prescrite
                flot(i,j)=.false.
                H(i,j)=(10.+sealevel_2d(i,j)-Bsoc(i,j))*row/ro  ! pour avoir archim=10
                new_flot_point(i,j)=.false.
             endif




         else                                       ! isynchro=0 ou il flottait dja
                      
            if (.not.FLOT(I,J)) then               ! il ne flottait pas (isynchro=0)
               new_flot_point(i,j)=.true.          ! signale un point qui ne flottait pas
                                                   ! au pas de temps precedent
                       flot(i,j)=.true.
 
                if (igrdline.eq.1) then   ! en cas de grounding line prescrite
                    flot(i,j)=.false.
                    H(i,j)=(10.+sealevel_2d(i,j)-Bsoc(i,j))*row/ro  ! pour avoir archim=10
                   new_flot_point(i,j)=.false.
               endif

            endif
         endif ex_pose


       else                           !    le point ne flotte pas
           if(FLOT(I,J)) then         !  mais il flottait avant
                FLOT(I,J)=.false.
                BOOST=.false.
           endif
       endif arch

!   Si la glace flotte -> PRUDENCE !!!
!   S et B sont alors determines avec la condition de flottabilite

             if (flot(i,j)) then
                shelfy = .true.

                surnet=H(i,j)*(1.-ro/row)
                S(i,j)=surnet+sealevel_2d(i,j)
                B(i,j)=S(i,j)-H(i,j)
             end if

          end do
       end do


      IF (BOOST) GOTO 11
!     vincent mars2004 on utilise plus boost, on redefini
!     le front a chaque pas de temps       
!     ou alors le mettre apres la determination du front
!     ou alors gardre la valeur des fronts : ne pas reinitialiser au
!     debut       



domain: do j=2,ny
         do i=2,nx

!    3    A- NOUVELLE DEFINITION DE FLOTMX, LES POINTS
!            AYANT UN DES VOISINS FLOTTANTS SONT FLOTMX ET GZMX
!         -------------------------------------------

!            fl1 est l'ancienne valeur de flotmx
             gz1=gzmx(i,j)         !     gz1 est l'ancienne valeur de gzmx
             fl1=flotmx(i,j)       !     fl1 est l'ancienne valeur de flotmx

             flotmx(i,j)=flot(i,j).and.flot(i-1,j)

! test pour detecter les nouveaux flotmx entre 2 dtt :

             if (flotmx(i,j).and.(new_flot_point(i,j).or. &
             new_flot_point(i-1,j))) then
                appel_new_flot=.true.
                new_flotmx(i,j)=.true.
             endif


!   determination de gzmx
!__________________________________________________________________________

! gzmx si un des deux voisins est flottant et l'autre pos
!                                                                   i-1   i
             gzmx(i,j)=((flot(i,j).and..not.flot(i-1,j)) & !         F    P
             .or.(.not.flot(i,j).and.flot(i-1,j)))         !         P    F

!                 A condition d'etre assez proche de la flottaison
!                 sur le demi noeud condition archim > 100 m

             archim=(Bsoc(i,j)+Bsoc(i-1,j))*0.5-sealevel_2d(i,j)+ro/row*Hmx(i,j)
             gzmx(i,j)=gzmx(i,j).and.(archim.le.100.) 

! mais gzmx peut tre quand mme vrai selon une conditon sur la pression effective 
             gzmx(i,j)=gzmx(i,j).or.((Neffmx(i,j).lt.neffratio*hmx(i,j))  &
                .and..not.flotmx(i,j))  



! ce cas est un ilemx trait plus loin
! if (i.gt.2.AND.i.lt.nx)   gzmx(i,j)=gzmx(i,j).or.(.not.flot(i,j).and. & 
!             .not.flot(i-1,j).and.flot(i+1,j).and.flot(i-2,j))     !    F    P    P    F


!      On rajoute une condition sur la contrainte basale
!      si tobmx(i,j) > toblim (de l'ordre de 1 bar), passage a pose

             gzmx(i,j)=gzmx(i,j).and.(abs(tobmx(i,j)).lt.toblim)
             gzmx(i,j)=gzmx(i,j).or.((hwater(i,j).gt.hwatstream).and..not.flotmx(i,j))

!-------------------------------------------------------------------------------------
! attention : pour expriences Heino
             gzmx(i,j)=gzmx_heino(i,j)
             shelfy=.true.
!             shelfy=.false.
!____________________________________________________________________________________ 
             


! ATTENTION : A METTRe A LA FIN EN FAISANT PORTER SUR ilmx,flotmx gzmx
!      si le point etait pos (non gz) et devient grounded
!      definir la direction de la vitesse (moyenne des points)

             if ((.not.gz1).and.(.not.fl1).and.gzmx(i,j).and. &
            (i.gt.2).and.(i.lt.nx))     then
                 uxs1(i,j)=(uxbar(i+1,j)+uxbar(i-1,j))/2.
             endif


!     3    B- NOUVELLE DEFINITION DE FLOTMY
!         --------------------------------


             gz1=gzmy(i,j)           !       gz1 est l'ancienne valeur de gzmy
             fl1=flotmy(i,j)         !       fl1 est l'ancienne valeur de flotmy

             flotmy(i,j)=flot(i,j).and.flot(i,j-1)

! test pour detecter les nouveaux flotmy entre 2 dtt :

             if (flotmy(i,j).and.(new_flot_point(i,j).or.     &
             new_flot_point(i,j-1))) then
                appel_new_flot=.true.
                new_flotmy(i,j)=.true.
             endif

!   determination de gzmy
!__________________________________________________________________________

! gzmy si un des deux voisins est flottant et l'autre pos

             gzmy(i,j)=((flot(i,j).and..not.flot(i,j-1))          &
             .or.(.not.flot(i,j).and.flot(i,j-1)))                 

!                 A condition d'etre assez proche de la flottaison
!                 sur le demi noeud condition archim > 100 m

             archim=(Bsoc(i,j)+Bsoc(i,j-1))*0.5-sealevel_2d(i,j)+ro/row*Hmy(i,j)
             gzmy(i,j)=gzmy(i,j).and.(archim.le.100.) 

! mais gzmx peut tre quand mme vrai selon une conditon sur la pression effective 
             gzmy(i,j)=gzmy(i,j).or.((Neffmy(i,j).lt.neffratio*hmy(i,j))  &
                .and..not.flotmy(i,j))

! ce cas est un ilemy trait plus loin
!          if (j.gt.2.AND.j.lt.ny) gzmy(i,j)=gzmy(i,j).or.(flot(i,j).and.      &
!             .not.flot(i,j-1).and.flot(i,j+1).and.flot(i,j-2))
               

!      On rajoute une condition sur la contrainte basale
!      si tobmy(i,j) > toblim (de l'ordre de 1 bar), passage a pose
             gzmy(i,j)=gzmy(i,j).and.(abs(tobmy(i,j)).lt.toblim)
             gzmy(i,j)=gzmy(i,j).or.((hwater(i,j).gt.hwatstream).and..not.flotmy(i,j))


!-------------------------------------------------------------------------------------
! attention : pour expriences Heino
             gzmy(i,j)=gzmy_heino(i,j)
             shelfy=.true.
!             shelfy=.false.
!____________________________________________________________________________________ 


! ATTENTION : A METTRe A LA FIN EN FAISANT PORTER SUR ilmx,flotmx gzmx
!      si le point etait pos (non gz) et devient grounded
!      definir la direction de la vitesse (moyenne des points)

          if (j.gt.2.AND.j.lt.ny) then
             if ((.not.gz1).and.(.not.fl1).and.gzmy(i,j))    then
             uys1(i,j)=(uybar(i,j+1)+uybar(i,j-1))/2.
             endif


! Points du shelf proches de la grounding line (pour la fusion basale)
!       fbm est vrai si le point est flottant mais un des voisins est pose 
!_________________________________________________________________________

             if (i.gt.2.AND.i.lt.nx) then    
             fbm(i,j)=flot(i,j).and.      & 
              ((.not.flot(i+1,j)).or.(.not.flot(i,j+1)) &
              .or.(.not.flot(i-1,j)).or.(.not.flot(i,j-1)))
            endif  
          endif

          end do
       end do domain


! Condition sur les bords
      
       do i=2,nx
         flotmx(i,1) = (flot(i,1).or.flot(i-1,1))
         flotmy(i,1) = .false. 
       end do
       
       do j=2,ny
          flotmy(1,j) = (flot(1,j).or.flot(1,j-1))
          flotmx(1,j) = .false. 
       end do

       flotmx(1,1) = .false.
       flotmy(1,1) = .false.

 11    continue


!      4- determination des iles
!      -------------------------
 
          ilemx(:,:)=.false.
          ilemy(:,:)=.false.

!       selon x 
         do j=2,ny-1
           do i=3,nx-2
!                       F   G   F   
!                           x
! modif tof 26/08/02 limite sur la pente (si diff S > 400 m)
            if ((flot(i-1,j).and..not.flot(i,j).and.flot(i+1,j)).and. &
                (sdx(i,j).LT.1.E-02)) then
             ilemx(i,j)=.true.
             ilemx(i+1,j)=.true.

!                      F  G   G   F
!                         x
! modif tof 26/08/02 limite sur la pente (si diff S > 400 m)
            else if ((flot(i-1,j).and..not.flot(i,j)                  &
                .and..not.flot(i+1,j)).and.flot(i+2,j).and.           &
                 (sdx(i,j).LT.1.E-02.and.sdx(i+1,j).LT.1.E-02)) then
              ilemx(i,j)=.true.
              ilemx(i+1,j)=.true.
              ilemx(i+2,j)=.true.

!                      F   G   G   F
!                              x
! modif tof 26/08/02 limite sur la pente (si diff S > 400 m)
            else if ((flot(i-2,j).and..not.flot(i-1,j)                &
                .and..not.flot(i,j)).and.flot(i+1,j).and.             &
                 (sdx(i,j).LT.1.E-02.and.sdx(i-1,j).LT.1.E-02)) then
              ilemx(i-1,j)=.true.
              ilemx(i,j)=.true.
              ilemx(i+1,j)=.true.

!                      F   G   G   G    F
!                              x
! modif tof 26/08/02 limite sur la pente (si diff S > 400 m)
            else if ((i.lt.nx-2)                                      &
                .and.(flot(i-2,j).and..not.flot(i-1,j)                &
                .and..not.flot(i,j)).and..not.flot(i+1,j)             &
                 .and.flot(i+2,j).and.                                &
                 (sdx(i,j).LT.1.E-02.and.sdx(i-1,j).LT.1.E-02         &
                 .and.sdx(i+1,j).LT.1.E-02)) then
              ilemx(i-1,j)=.true.
              ilemx(i,j)=.true.
              ilemx(i+1,j)=.true.
              ilemx(i+2,j)=.true.

            endif

           end do
          end do

!       selon y 
         do j=3,ny-2
           do i=2,nx-1
!                       F   G   F   
!                           x
! modif tof 26/08/02 limite sur la pente (si diff S > 400 m)
            if ((flot(i,j-1).and..not.flot(i,j).and.flot(i,j+1)).and. &
                 (sdy(i,j).LT.1.E-02)) then
             ilemy(i,j)=.true.
             ilemy(i,j+1)=.true.

!                      F  G   G   F
!                         x
! modif tof 26/08/02 limite sur la pente (si diff S > 400 m)
            else if ((flot(i,j-1).and..not.flot(i,j)                  &
                .and..not.flot(i,j+1)).and.flot(i,j+2).and.           &
                 (sdy(i,j).LT.1.E-02.and.sdy(i,j+1).LT.1.E-02)) then
              ilemy(i,j)=.true.
              ilemy(i,j+1)=.true.
              ilemy(i,j+2)=.true.

!                      F   G   G   F
!                              x
! modif tof 26/08/02 limite sur la pente (si diff S > 400 m)
            else if ((flot(i,j-2).and..not.flot(i,j-1)                 &
                .and..not.flot(i,j)).and.flot(i,j+1).and.              &
                 (sdy(i,j).LT.1.E-02.and.sdy(i,j-1).LT.1.E-02)) then
              ilemy(i,j-1)=.true.
              ilemy(i,j)=.true.
              ilemy(i,j+1)=.true.

!                      F   G   G   G    F
!                              x
! modif tof 26/08/02 limite sur la pente (si diff S > 400 m)
            else if ((j.lt.ny-2)                                      &
                .and.(flot(i,j-2).and..not.flot(i,j-1)                &
                .and..not.flot(i,j)).and..not.flot(i,j+1)             &
                 .and.flot(i,j+2).and.                                &
                 (sdy(i,j).LT.1.E-02.and.sdy(i,j-1).LT.1.E-02         &
                 .and.sdy(i,j+1).LT.1.E-02)) then 
              ilemy(i,j-1)=.true.
              ilemy(i,j)=.true.
              ilemy(i,j+1)=.true.
              ilemy(i,j+2)=.true.
            endif
           end do
          end do


!         les iles n'ont pas de condition neff mais ont la condition toblim
            do j=2,ny
             do i=2,nx
                ilemx(i,j)=ilemx(i,j).and.(abs(tobmx(i,j)).lt.toblim)
                ilemy(i,j)=ilemy(i,j).and.(abs(tobmy(i,j)).lt.toblim)
             end do 
            end do 

!  6-On determine finalement la position des noeuds stream/shelf
!  -------------------------------------------------------------

   do j=2,ny-1
      do i=2,nx-1

        if (nt.ge.2) then   ! pour ne pas faire ce calcul lors du premier passage
 
          uxbar(i,j)=uxs1(i,j)
          uybar(i,j)=uys1(i,j)
        endif

        FLGZMX(I,J)=(marine.and.(flotmx(i,j).or.gzmx(i,j).or.ilemx(i,j)))   &
          .or.(.not.marine.and.flotmx(i,j))
        FLGZMY(I,J)=(marine.and.(flotmY(i,j).or.gzmY(i,j).or.ilemy(i,j)))   &
          .or.(.not.marine.and.flotmy(i,j))

      end do
    end do

!  5-On determine maintenant la position du front de glace
!  -------------------------------------------------------
!  C'est ici que l'on determine la position et le type de front
!       print*,'on est dans flottab pour definir les fronts'
      

      DO I=3,NX-2
       DO J=3,NY-2
         IF (H(i,j).gt.1.1) ICE(i,j)=1        
       END DO
      END DO
!print*,'H ice 50,30',H(50,30),ICE(50,30)

!open(UNIT=num_file4,file='TABLE_ice-big')
!       do i=1,nx
!         do j=1,ny
!           write(num_file4,*) i,j,ice(i,j)
!         enddo
!       enddo
!close (num_file4)
    if (ISYNCHRO.eq.1) then
!----------------------------------------------!
!On determine les differents ice strean/shelf  !
      call DETERMIN_TACHE                      !
!----------------------------------------------!
!           do i=1,nx
!           do j=1,ny
!           if (ice(i,j)==1.and.table_out(i,j)==0) then
!           print*,'i,j AAAAAAAAAAAAAAA',i,j
!           endif
!           enddo
!           enddo
      ICE=0
!On compte comme englac uniquement les calottes dont une partie et pos      
!          pause
!      DO I=2,NX-1
!       DO J=2,NY-1
      DO I=3,NX-2
       DO J=3,NY-2
        IF (.not.iceberg(table_out(i,j))) THEN !!! on est pas sur un iceberg TEST1
                if (nb_pts_tache(table_out(i,j)).ge.1) then
                        ICE(i,j)=1
                        if (nb_pts_tache(table_out(i,j)).le.5) then
                                FLGZMX(I,J)=.false.
                                FLGZMY(I,J)=.false.
!                               print*,'i,jcoups',FLGZMX(I,J),FLGZMY(I,J),i,j
                        endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ici on est sur une tache non iceberg >= 5 points                       
!!!!!!!!!                      on teste si la tache n'est pas completement ice stream
      if (icetrim(table_out(i,j))) THEN ! on a une ile d'ice stream   !!!! TEST2
              smax_i=0 ; smax_j=0 ; smax_=sealevel !afq -- warning: what about local sealevel?
              DO II=3,NX-2
                DO JJ=3,NY-2
                if (table_out(ii,jj).eq.table_out(i,j)) then
                        if (S(ii,jj).gt.smax_) then
                                smax_ =S(ii,jj)
                                smax_i=ii
                                smax_j=jj
                        endif     
                endif     
                END DO
              END DO
              gzmx(smax_i,smax_j)=.false. ; gzmx(smax_i+1,smax_j)=.false.
              gzmy(smax_i,smax_j)=.false. ; gzmx(smax_i,smax_j+1)=.false.
              flgzmx(smax_i,smax_j)=.false. ; flgzmx(smax_i+1,smax_j)=.false.
              flgzmy(smax_i,smax_j)=.false. ; flgzmx(smax_i,smax_j+1)=.false.
                 endif !!!                                       !!!!!!!   TEST2
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!                 
         ELSE !!! on est sur un iceberg                          !!!!!!!!   TEST1
                 ICE(i,j)=0
!                print*,'iceberg',i,j,h(i,j)
                 H(i,j)=1.
         ENDIF        !!!!!!!!!!!!                               !!!!!!!!   TEST1
     endif           
!     IF (.not.iceberg(table_out(i,j)).and.nb_pts_tache(table_out(i,j)).ge.1) THEN
!           ICE(i,j)=1
!           if (nb_pts_tache(table_out(i,j)).le.5) then
!              FLGZMX(I,J)=.false.
!              FLGZMY(I,J)=.false.
!print*,'i,j coups',FLGZMX(I,J),FLGZMY(I,J),i,j
!           endif
!      ELSE
!IF (table_out(i,j).gt.0) print*,'i,j coups',i,j,table_out(i,j),nb_pts_tache(table_out(i,j)),flgzmx(i,j),flotmx(i,j)
!           ENDIF
       END DO
      END DO

!call printtable_i(ice,nom_table)         
!ause
!----------------------------------------------!
!On caracterise le front des ice shelfs/streams!
      call DETERMIN_FRONT                      !
!----------------------------------------------!      

         
!nom_table='ice_af'
!call printtable_i(ice,nom_table)         
!nom_table='GZMX2'
!call printtable_l(FLGZMX,nom_table)
!nom_table='flot'
!call printtable_l(FLOT,nom_table)
!nom_table='mk'
!call printtable_i(mk,nom_table)
!nom_table='mk0'
!call printtable_i(mk0,nom_table)
    endif


!fin de routine flottab2
!print*, 'front',front(50,30),ice(50,30),flotmx(i,j),uxbar(i,j)
contains            
!====================================================================
!====================================================================
      SUBROUTINE DETERMIN_TACHE
      implicit none
      integer :: indice
      integer :: label         ! no des taches rencontres dans le mask
      integer :: label_max     ! no temporaire maxi de tache rencontres
!      integer :: mask_nb = 4
      integer :: mask_nb = 2   ! version ou on ne compte pas les diagonales
      integer :: vartemp       ! variable temporaire pour reorganiser compt
!     integer,dimension(mask_nb) :: mask
      integer,dimension(2) :: mask

! 1-initialisation
!-----------------
      label_max=1      ! numero de la tache, la premiere tache est note 1
      label=1
      do i=1,n_ta_max
      compt(i)=i
      enddo
!      table_in  = .false.
      table_out = 0
      iceberg = .true.
      icetrim = .true.
      nb_pts_tache = 0
!    open(unit=100,file="tache.data",status='replace')

! 2-reperage des taches
!----------------------
do i=1,nx
  do j=1,ny

     if (label_max==0) print*,'labmax=0',i,j
  IF (ice(i,j).ge.1) THEN ! on est sur la glace-----------------------------!
!   if ((ice(i-1,j-1).ge.1).or.(ice(i-1,j).ge.1).or.&    !masque de 4 cases adjacentes
!         (ice(i-1,j+1).ge.1).or.(ice(i,j-1).ge.1))  then
!         mask(1) = table_out(i-1,j-1)
!         mask(2) = table_out(i-1,j)
!         mask(3) = table_out(i-1,j+1)
!         mask(4) = table_out(i,j-1)
      if ((ice(i-1,j).ge.1).or.(ice(i,j-1).ge.1)) then   !masque de 2 cases adjacentes
!      un des voisins est deja en glace
         mask(1) = table_out(i-1,j)
         mask(2) = table_out(i,j-1)
           label = label_max

!on determine la valeur de la tache minimun (>0) presente ds le masque
            do indice=1,mask_nb
              if (mask(indice).gt.0) label=min(label,mask(indice))
            enddo

!on fixe la valeur de la tache voisine minimun au point etudi(via label)
         table_out(i,j)=label
!si ce noeud est pos, alors le tache n'est pas un iceberg et iceberg=.F.
             if (.not.FLOT(I,J)) then
             iceberg(label)=.false.
             endif
!si ce noeud est pos, alors la tache n'est pas un ice stream et icestrim=.F.
             if (.not.gzmx(I,J).AND..not.gzmy(I,J)) then
             icetrim(label)=.false.
             endif

!si 2 taches differentes sont dans le masque, il faut les identifier dans compt
!i.e. les plus grands numeros correspondent au plus petit 
!on lui affecte le numero de la tache fondamentale avec un signe - pour indiquer le changement
              do indice=1,mask_nb
                if(mask(indice).gt.label) then
                   compt(mask(indice))=-label
                endif
              enddo

       else !aucun des voisins est une tache
                table_out(i,j)= label_max
                compt(label_max)=label_max
!si ce noeud est pos, alors la ache n'est pas un iceberg et iceberg=.F.
              if (.not.FLOT(I,J)) then
              iceberg(label_max)=.false.
              endif
!si ce noeud est pos, alors le tache n'est pas un ice stream et icestrim=.F.
             if (.not.gzmx(I,J).AND..not.gzmy(I,J)) then
             icetrim(label)=.false.
             endif
             
              label_max  = label_max+1
              if (label_max.gt.n_ta_max) print*,'trop de taches=',label_max 
        endif 

ELSE !on est pas sur une tache----------------------------------------------!
        table_out(i,j)=0    ! Pas necessaire (reecrit 0 sur 0)              !
ENDIF!----------------------------------------------------------------------!


  enddo
enddo
          
! On reorganise compt en ecrivant le numero de la tache fondamentale
! i.e. du plus petit numero present sur la tache (Sans utiliser de recursivit)
! On indique aussi le nb de point que contient chaque taches (nb_pts_tache)        
      do indice=1,label_max
      vartemp = compt(indice)
       if (compt(indice).lt.0) then 
               compt(indice)= compt(-vartemp)
               if (.not.iceberg(indice)) iceberg(-vartemp)=.false.
               if (.not.icetrim(indice)) icetrim(-vartemp)=.false.
      endif
      
      enddo    
   do i=1,nx
     do j=1,ny
       if (table_out(i,j).ne.0) then
       table_out(i,j)=compt(table_out(i,j))  
       nb_pts_tache(compt(table_out(i,j)))= nb_pts_tache(compt(table_out(i,j)))+1   
       endif
     enddo
   enddo
! 3-visualisation  
!----------------
!print*,'_pts_tache0:',nb_pts_tache(0:label_max+15)
!print*,'lab tache', label_max      
!print*,'compt',compt      
!om_table='tache'
!call printtable_i(table_out,nom_table)
!nom_table='bergs'
!call printtable_l(iceberg(table_out),nom_table)
!nom_table='pt_ta'
!call printtable_i(nb_pts_tache(table_out),nom_table)
     
     
END SUBROUTINE DETERMIN_TACHE
!====================================================================
!====================================================================       
       SUBROUTINE DETERMIN_FRONT
      
      DO I=3,NX-2
       DO J=3,NY-2

 surice:IF  (ICE(i,j)==0) THEN
         do pmx=-1,1,2
          do pmy=-1,1,2
 diagice :  if (ICE(i+pmx,j+pmy)==1) then
!!!!!     print*,'========',i,j,ICE(i+pmx,j)+ICE(i,j+pmy)
              if ((ICE(i+pmx,j)+ICE(i,j+pmy)==2)) then
! test (i) pour eviter les langues de glaces diagonales en coin(26dec04)
                   if ((ICE(i+2*pmx,j)==1.and.ICE(i+2*pmx,j+pmy)==0).or.&
                       (ICE(i,j+2*pmy)==1.and.ICE(i+pmx,j+2*pmy)==0)) then
!                      print*,i,j,'langues en coin'
                           ICE(i,j)=1
                           H(i,j)=max(1.,H(i,j))
                   endif         
! test (i) pour eviter les langues de glaces diagonales
! mouvement du cheval aux echecs  
               if ((ICE(i+2*pmx,j+pmy)+ICE(i+pmx,j+2*pmy)==1)) then
                   if (ICE(i+2*pmx,j+pmy)==1.and. &
                      (ICE(i+2*pmx,j+2*pmy)+ICE(i,j+2*pmy)).ge.1) then
                           ICE(i,j)=1
                           H(i,j)=max(1.,H(i,j))
                   endif        
                   if (ICE(i+pmx,j+2*pmy)==1.and. &
                      (ICE(i+2*pmx,j+2*pmy)+ICE(i+2*pmx,j)).ge.1) then
                           ICE(i,j)=1
                           H(i,j)=max(1.,H(i,j))
                   endif        
! test (ii) pour eviter les langues de glaces diagonales
! le point glace ICE(i+pmx,j+pmy) a ses 4 voisins frontaux en glace
! mais 2 voisins vides diagonalement opposes               
               elseif ((ICE(i+2*pmx,j+pmy)+ICE(i+pmx,j+2*pmy)==2)  &
                                .and.ICE(i+2*pmx,j+2*pmy)==0) then
! test (iii) pour faire les tests (i) et (ii)
!                  if ((ICE(i+2*pmx,j+pmy)+ICE(i+pmx,j+2*pmy).ge.1)  &
!                              .and.ICE(i+2*pmx,j+2*pmy)==0) then
                    ICE(i,j)=1
                    H(i,j)=max(1.,H(i,j))
                    ICE(i+2*pmx,j+2*pmy)=1
                    H(i+2*pmx,j+2*pmy)=max(1.,H(i+2*pmx,j+2*pmy))
               endif
            endif
           endif diagice   
          enddo
         enddo
         ENDIF surice       
       END DO
      END DO
!     print*,'dans remplissage baies',time
      DO I=3,NX-2
       DO J=3,NY-2
 surice2:IF  (ICE(i,j)==0) THEN
! test (iii) pour trouver les baies de largeur 1 ou 2 cases 
! et combler les trous si ce sont des baies
! si ce ne sont pas des baies, ne pas combler et creer des langues de glaces artificielles             
! baies horizontales  
        if (ICE(i-1,j)==1.and.(ICE(i+1,j)==1.or.ICE(i+2,j)==1)) then 
            if (ICE(i,j-1)==1.or.ICE(i,j+1)==1) then! ICE(i,j)=1
               ICE(i,j)=1
               H(i,j)=max(1.,H(i,j))
               endif
        endif
! baies verticales
        if (ICE(i,j-1)==1.and.(ICE(i,j+1)==1.or.ICE(i+2,j)==1)) then 
            if (ICE(i-1,j)==1.or.ICE(i+1,j)==1) then !ICE(i,j)=1
               ICE(i,j)=1
               H(i,j)=max(1.,H(i,j))
               endif
!               ICE(i,j)=1
        endif         
!     if (ICE(i-1,j)==1.and.(ICE(i+1,j)==1.or.ICE(i+2,j)==1)) ICE(i,j)=1
!     if (ICE(i,j-1)==1.and.(ICE(i,j+1)==1.or.ICE(i+2,j)==1)) ICE(i,j)=1
         ENDIF surice2       
       END DO
      END DO
       DO I=3,NX-2
        DO J=3,NY-2 
 surice3:IF  (ICE(i,j)==0) THEN
!! test (iii) pour trouver les baies de largeur 1 ou 2 cases (oublie au test d'avant)
!! et combler les trous si ce sont des baies
!! si ce ne sont pas des baies, ne pas combler et creer des langues de glaces artificielles  
!! baies horizontales  
        if (ICE(i-1,j)==1.and.(ICE(i+1,j)==1.or.ICE(i+2,j)==1)) then
             if (ICE(i,j-1)==1.or.ICE(i,j+1)==1) then!ICE(i,j)=1
                ICE(i,j)=1
                H(i,j)=max(1.,H(i,j))
!               if (ISYNCHRO.eq.1) print*,'surice3 hor',i,j
                endif
!                ICE(i,j)=1
         endif
!! baies verticales
         if (ICE(i,j-1)==1.and.(ICE(i,j+1)==1.or.ICE(i+2,j)==1)) then
             if (ICE(i-1,j)==1.or.ICE(i+1,j)==1) then!ICE(i,j)=1
                ICE(i,j)=1
                H(i,j)=max(1.,H(i,j))
!               if (ISYNCHRO.eq.1) print*,'surice3 ver',i,j
              endif
                                                
!                ICE(i,j)=1
         endif         
!!     if (ICE(i-1,j)==1.and.(ICE(i+1,j)==1.or.ICE(i+2,j)==1)) ICE(i,j)=1
!!     if (ICE(i,j-1)==1.and.(ICE(i,j+1)==1.or.ICE(i+2,j)==1)) ICE(i,j)=1
          ENDIF surice3       
        END DO
       END DO

     DO I=1,NX
       DO J=1,NY

        IF (ICE(i,j)==1) THEN     !!!!!!!!!!!!!!!!!!test SI ICE=1
! if ICE, on determine FRONT...
! ainsi, FRONT=0 sur les zones = 0
         FRONT(i,j)=(ICE(i-1,j)+ICE(i+1,j)+ICE(i,j+1)+ICE(i,j-1))
!front= le nb de faces en contact avec un voisin englac
        ENDIF
       END DO
      END DO

     DO I=1,NX
       DO J=1,NY
!On ne compte pas les icebergs de 2 cases (horizontales ou verticales)
       IF (FRONT(i,j).eq.1) then
         IF (FRONT(i+1,j).eq.1) then
          ICE(i,j)=0
          ICE(i+1,j)=0
          FRONT(i,j)=0
          FRONT(i+1,j)=0
         ENDIF
         IF (FRONT(i,j+1).eq.1) then
          ICE(i,j)=0
          ICE(i,j+1)=0
          FRONT(i,j)=0
          FRONT(i,j+1)=0
         ENDIF
        ENDIF

       IF (FRONT(i,j).ge.1.and.FRONT(i,j).le.3) THEN !front(entre 1 et 3)

            if ((ICE(i-1,j)+ICE(i+1,j)).lt.2) then
              ! il y a un front // a x
              if ((ICE(i-1,j)+ICE(i+1,j)).eq.0) then
                      isolx(i,j)=.TRUE.
              elseif (ICE(i-1,j).eq.0) then
                      frontfacex(i,j)=-1    ! front  i-1 |i  i+1
              else
                      frontfacex(i,j)=+1    ! front  i-1  i| i+1
              endif
            endif


            if ((ICE(i,j-1)+ICE(i,j+1)).lt.2) then
              ! il y a un front // a y
              if ((ICE(i,j-1)+ICE(i,j+1)).eq.0) then
                      isoly(i,j)=.TRUE.    !front   j-1 |j| j+1
              elseif (ICE(i,j-1).eq.0) then
                      frontfacey(i,j)=-1   !front   j-1 |j j+1
              else
                      frontfacey(i,j)=+1   !front   j-1  j| j+1
              endif
            endif
!isolx signifie pas de voisins en x
!isoly signifie pas de voisins en y
!Remarque : 
!si isolx/y=.true. alors frontfacex/y=0 (a la fois +1 & -1 or +1-1=0)

       END IF !fin du test il y a un front
!!!!!!!!!!!!!!!!      END IF  !!!!!!!!!!!!!!!!!!fin du test SI ICE=1
      END DO
   END DO
END SUBROUTINE DETERMIN_FRONT       
!====================================================================
!====================================================================       
      
END SUBROUTINE FLOTTAB

